@@ -19,6 +19,10 @@ struct ScreenInfo
   SizeID current_size;
   short current_rate;
   Rotation current_rotation;
+
+  SizeID old_size;
+  short old_rate;
+  Rotation old_rotation;
   
   XRRScreenConfiguration *config;
   XRRScreenSize *sizes;
@@ -73,6 +77,40 @@ read_display_info (GdkDisplay *display)
   return info;
 }
 
+void
+update_display_info (struct DisplayInfo *info, GdkDisplay *display)
+{
+  struct ScreenInfo *screen_info;
+  GdkScreen *screen;
+  GdkWindow *root_window;
+  int i;
+
+  g_assert (info->n_screens == gdk_display_get_n_screens (display));
+  
+  for (i = 0; i < info->n_screens; i++)
+    {
+      screen = gdk_display_get_screen (display, i);
+      
+      screen_info = &info->screens[i];
+
+      screen_info->old_rate = screen_info->current_rate;
+      screen_info->old_size = screen_info->current_size;
+      screen_info->old_rotation = screen_info->current_rotation;
+      
+      screen_info->current_width = gdk_screen_get_width (screen);
+      screen_info->current_height = gdk_screen_get_height (screen);
+
+      root_window = gdk_screen_get_root_window (screen);
+      XRRFreeScreenConfigInfo (screen_info->config);
+      screen_info->config = XRRGetScreenInfo (gdk_x11_display_get_xdisplay (display),
+					      gdk_x11_drawable_get_xid (GDK_DRAWABLE (root_window)));
+
+      screen_info->current_rate = XRRConfigCurrentRate (screen_info->config);
+      screen_info->current_size = XRRConfigCurrentConfiguration (screen_info->config, &screen_info->current_rotation);
+      screen_info->sizes = XRRConfigSizes (screen_info->config, &screen_info->n_sizes);
+    }
+}
+
 static int
 get_current_resolution (struct ScreenInfo *screen_info)
 {
@@ -150,7 +188,14 @@ apply_config (struct DisplayInfo *info)
 					      GDK_CURRENT_TIME);
 	}
     }
+
+  update_display_info (info, display);
   
+  /* xscreensaver should handle this itself, but does not currently so we hack
+   * it.  Ignore failures in case xscreensaver is not installed */
+  if (changed)
+   g_spawn_command_line_async ("xscreensaver-command -restart", NULL);
+
   return changed;
 }
 
@@ -177,14 +222,28 @@ revert_config (struct DisplayInfo *info)
       status = XRRSetScreenConfigAndRate (xdisplay, 
 					  screen_info->config,
 					  gdk_x11_drawable_get_xid (GDK_DRAWABLE (root_window)),
-					  screen_info->current_size,
-					  screen_info->current_rotation,
-					  screen_info->current_rate,
+					  screen_info->old_size,
+					  screen_info->old_rotation,
+					  screen_info->old_rate,
 					  GDK_CURRENT_TIME);
-
-		generate_resolution_menu(screen_info);
-		generate_rate_menu(screen_info);
+      
     }
+
+  update_display_info (info, display);
+
+  /* Need to update the menus to the new settings */
+  for (i = 0; i < info->n_screens; i++)
+    {
+      struct ScreenInfo *screen_info = &info->screens[i];
+      
+      generate_resolution_menu (screen_info);
+      generate_rate_menu (screen_info);
+    }
+
+  /* xscreensaver should handle this itself, but does not currently so we hack
+   * it.  Ignore failures in case xscreensaver is not installed */
+  g_spawn_command_line_async ("xscreensaver-command -restart", NULL);
+  
   return 0;
 }
 
@@ -431,6 +490,7 @@ create_dialog (struct DisplayInfo *info)
   GtkWidget *per_computer_check;
   int i;
   GtkWidget *wrapped;
+  GtkWidget *vbox;
   GConfClient *client;
   char *key;
   char *resolution;
@@ -453,15 +513,20 @@ create_dialog (struct DisplayInfo *info)
 					NULL);
 					
   gtk_window_set_resizable(GTK_WINDOW (dialog), FALSE);
-  gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
+  gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);  
   gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
   capplet_set_icon (dialog, "display-capplet.png");
   
+  vbox = gtk_vbox_new (FALSE, 18);
+  gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+                      vbox, FALSE, FALSE, 0);
+  gtk_widget_show (vbox);
+  
   for (i = 0; i < info->n_screens; i++)
     {
       screen_widget = create_screen_widgets (&info->screens[i], i, info->n_screens == 1);
-      gtk_container_set_border_width (GTK_CONTAINER (screen_widget), 5);
-      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+      gtk_box_pack_start (GTK_BOX (vbox),
 			  screen_widget, FALSE, FALSE, 0);
       gtk_widget_show (screen_widget);
     }
@@ -492,7 +557,7 @@ create_dialog (struct DisplayInfo *info)
       gtk_widget_show (per_computer_check);
       
       wrapped = wrap_in_label (per_computer_check, _("Options"));
-      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+      gtk_box_pack_start (GTK_BOX (vbox),
 			  wrapped, FALSE, FALSE, 0);
       gtk_widget_show (wrapped);
     }
@@ -684,9 +749,7 @@ cb_dialog_response (GtkDialog *dialog, gint response_id, struct DisplayInfo *inf
       gtk_main_quit ();
       break;
     case GTK_RESPONSE_HELP:
-      /* FIXME: This needs to be changed to the right section
-       * when the docs have been written. */
-		capplet_help (GTK_WINDOW (dialog), "user-guide.xml", "goscustdesk-38");
+      capplet_help (GTK_WINDOW (dialog), "user-guide.xml", "goscustdesk-70");
       break;
     case GTK_RESPONSE_APPLY:
       save_computer = info->per_computer_check != NULL && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (info->per_computer_check));
