@@ -27,6 +27,7 @@
 #include <gnome.h>
 #include <libgnome/gnome-desktop-item.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <libgnomevfs/gnome-vfs.h>
 
 #define CAPPLET_DIR_ENTRY(obj) ((CappletDirEntry *) obj)
 #define CAPPLET_DIR(obj) ((CappletDir *) obj)
@@ -52,9 +53,8 @@ struct _CappletDirEntry
 	GnomeDesktopItem *entry;
 	gchar **exec;
 	gchar *label;
-	gchar *icon;
-	gchar *path;
-	GdkPixbuf *pb;
+        GnomeVFSURI *uri;
+	GdkPixbuf *icon;
 	CappletDir *dir;	
 };
 
