@@ -124,7 +124,11 @@ static TranslationEntry translations [] = {
   { "/desktop/gnome/interface/gtk-im-preedit-style",	"Gtk/IMPreeditStyle",
       GCONF_VALUE_STRING,	translate_string_string },
   { "/desktop/gnome/interface/gtk-im-status-style",	"Gtk/IMStatusStyle",
-      GCONF_VALUE_STRING,	translate_string_string }
+      GCONF_VALUE_STRING,	translate_string_string },
+  { "/desktop/gnome/interface/icon_theme",		"Net/IconThemeName",
+      GCONF_VALUE_STRING,	translate_string_string },
+  { "/desktop/gnome/interface/file_chooser_backend",	"Gtk/FileChooserBackend",
+      GCONF_VALUE_STRING,	translate_string_string },
 };
 
 static TranslationEntry*
@@ -276,7 +280,7 @@ gnome_xft_settings_get (GConfClient      *client,
   settings->antialias = TRUE;
   settings->hinting = TRUE;
   settings->hintstyle = "hintfull";
-  settings->dpi = 96;
+  settings->dpi = 96 * 1024;
   settings->rgba = "rgb";
 
   if ((int)(1024 * dpi + 0.5) > 0)
